<?php
/**
 * @package MiGCMS
 * @author Justin C. Klein Keane <justin@madirish.net>
 * @version 2.0
 *
 * @abstract  DB_Object is a wrapper class to MDB2.  It provides some custom functionality, but mainly centralizes the interface and error handling for connections to the data layer.
 *
 * @todo Alter construct_error() so that errors are logged and present in a more friendly manner.
 */

require_once('MDB2.php');


Class DB_Object extends MDB2 {

	/**
	 * This is the variable for holding the persistent data connection
	 *
	 * @var connection
	 */
	var $conn;

	public function __construct() {

		/**
		 * These connection string variables are pulled from conf/config.ini by /lib/util/parse_config.class.php
		 */
		$dbtype = $GLOBALS['database']['dbtype'];
		$dbuser = $GLOBALS['database']['dbuser'];
		$dbpass = $GLOBALS['database']['dbpass'];
		$dbhost = $GLOBALS['database']['dbhost'];
		$db = $GLOBALS['database']['db'];

		$conn_string = $dbtype . '://' . $dbuser . ':' . $dbpass . '@' . $dbhost . '/' . $db;
		$options = array(
			'debug'	=> 0
		);

		$this->conn = $this->connect($conn_string, $options);

		if ($this->isError($this->conn)) {
			print_r($this->errorMessage());
			die('MiGCMS Internal Error!');
		}
	}

	/**
	 * Tear down the connection.
	 *
	 */
	function close() {
		//clean up the connection
		$this->conn->disconnect();
	}

	/**
	 * This function is passed a failed database interaction object and constructs a useful error message out of it.
	 *
	 * @param object $err_obj
	 * @return srting $error
	 */
	private function construct_error($err_obj) {
		$error = '<h2>' . $err_obj->message . '</h2>';
		$error .= '<h3>' . $err_obj->code . '</h3>';
		$error .= '<pre>' . $err_obj->userinfo . '</pre>';
		die($error);
		//return $error;
	}

	/**
	 * Loop over the arrays and delete all the records from the $tables using the appropraite $primary_key and $id matches.
	 *
	 * @param array $tables
	 * @param array $primary_keys
	 * @param array $ids
	 */
	function delete($tables, $primary_keys, $ids) {
		if (! is_array($tables) || ! (is_array($primary_keys)) || ! (is_array($ids))) {
			die("All inputs must be arrays.");
		}
		elseif (!(count($tables) === count($primary_keys)) || !(count($primary_keys) === count($ids)) ) {
			die("Array counts must all be the same.");
		}
	}

	/**
	 * This is merely a wrapper function for MDB2::query
	 *
	 * @param string $sql
	 */
	function exec($sql) {
		$res = $this->conn->query($sql);
	}

	/**
	 * This function executes the input SQL and returns an associate array of the result set.
	 *
	 * @param string $sql
	 * @return array
	 */
	function get_all($sql) {
		$res = $this->conn->query($sql);
		$retval = $res->fetchAll(MDB2_FETCHMODE_ASSOC);
		if (isset($retval->message)) {
				$error = $this->construct_error($retval);
				return $error;
		}
		else {
				return $retval;
		}
	}

	/**
	 * This function executes a prepared statement.
	 *
	 * @param string $statement
	 * @param array $types
	 * @param array $data_array
	 * @return array
	 */
	public function prep_statement($statement, $types, $data_array) {
		if (count($types) != count($data_array)) {
			die('Problem in call to DB_Object::prep_statement - types number (' . count($types) . ') doesn\'t match data_array count(' . count($data_array) . ') for statement:<br/><pre>' . $statement . '</pre>');
		}
		$res = $this->conn->prepare($statement,$types, MDB2_PREPARE_MANIP) or die('Connection preparation failed.');
		$retval = $res->execute($data_array);
		if (isset($retval->message)) {
				$error = $this->construct_error($retval);
				return $error;
		}
		else {
				return $retval;
		}
	}
}

?>